From 0d3222d44fbf15c84c833664a1056d81e4ee7c0a Mon Sep 17 00:00:00 2001
From: Carl Eugen Hoyos <ceffmpeg@gmail.com>
Date: Sun, 6 Nov 2022 22:49:58 +0100
Subject: [PATCH] lavc/tiff: Support dng cropping

Reads cropping information from samples mentioned in ticket #4364.
---
 libavcodec/tiff.c | 65 +++++++++++++++++++++++++++++++++++++++++++++++
 1 file changed, 65 insertions(+)

diff --git a/libavcodec/tiff.c b/libavcodec/tiff.c
index fd9db18c0b..47e736d015 100644
--- a/libavcodec/tiff.c
+++ b/libavcodec/tiff.c
@@ -1250,6 +1250,30 @@ static void set_sar(TiffContext *s, unsigned tag, unsigned num, unsigned den)
     }
 }
 
+static int handle_crop(TiffContext *s, int count, int type, int *value, int *value2, const char *crop_type)
+{
+        if (count != 2 || type != TIFF_SHORT && type != TIFF_LONG && type != TIFF_RATIONAL) {
+            av_log(s->avctx, AV_LOG_WARNING, "Invalid crop %s (count: %d, type: %d)\n", crop_type, count, type);
+            return -1;
+        }
+        if (type == TIFF_RATIONAL) {
+            unsigned denum1, denum2;
+            *value  = ff_tget(&s->gb, TIFF_LONG, s->le);
+            denum1  = ff_tget(&s->gb, TIFF_LONG, s->le);
+            *value2 = ff_tget(&s->gb, TIFF_LONG, s->le);
+            denum2  = ff_tget(&s->gb, TIFF_LONG, s->le);
+            if (denum1 != 1 || denum2 != 1) {
+                av_log(s->avctx, AV_LOG_WARNING, "Unsuported crop %s\n", crop_type);
+                return -1;
+            }
+        } else {
+            *value  = ff_tget(&s->gb, type, s->le);
+            *value2 = ff_tget(&s->gb, type, s->le);
+        }
+        av_log(s->avctx, AV_LOG_DEBUG, "dng crop %s: %d/%d\n", crop_type, *value, *value2);
+        return 0;
+}
+
 static int tiff_decode_tag(TiffContext *s, AVFrame *frame)
 {
     AVFrameSideData *sd;
@@ -1492,6 +1516,47 @@ static int tiff_decode_tag(TiffContext *s, AVFrame *frame)
     case DNG_WHITE_LEVEL:
         s->white_level = value;
         break;
+    case DNG_CROP_ORIGIN:
+        if (handle_crop(s, count, type, &value, &value2, "origin") < 0)
+            break;
+        if (value >= s->width || value2 >= s->height) {
+            av_log(s->avctx, AV_LOG_WARNING, "Invalid crop origin (%d/%d)\n", value, value2);
+            break;
+        }
+        frame->crop_left = value;
+        frame->crop_top = value2;
+        break;
+    case DNG_CROP_SIZE:
+        if (handle_crop(s, count, type, &value, &value2, "size") < 0)
+            break;
+        if (value >= s->width - frame->crop_left || value2 >= s->height - frame->crop_top) {
+            av_log(s->avctx, AV_LOG_WARNING, "Invalid crop size (%d/%d)\n", value, value2);
+            break;
+        }
+        frame->crop_right = s->width - frame->crop_left - value;
+        frame->crop_bottom = s->height - frame->crop_top - value2;
+        break;
+    case DNG_ACTIVE_AREA: {
+        unsigned top, left, bottom, right;
+        if (count != 4 || type != TIFF_SHORT && type != TIFF_LONG) {
+            av_log(s->avctx, AV_LOG_WARNING, "Invalid Active Area (count: %d, type: %d)\n", count, type);
+            break;
+        }
+        top    = ff_tget(&s->gb, type, s->le);
+        left   = ff_tget(&s->gb, type, s->le);
+        bottom = ff_tget(&s->gb, type, s->le);
+        right  = ff_tget(&s->gb, type, s->le);
+        if (top >= s->height - frame->crop_top || top > frame->crop_bottom || left >= s->width - frame->crop_left || left > frame->crop_right) {
+            av_log(s->avctx, AV_LOG_WARNING, "Invalid Active Area (%d/%d/%d/%d)\n", top, left, bottom, right);
+            break;
+        }
+        av_log(s->avctx, AV_LOG_DEBUG, "dng active area %d / %d / %d / %d\n", top, left, bottom, right);
+        frame->crop_top += top;
+        frame->crop_bottom -= top;
+        frame->crop_left += left;
+        frame->crop_right -= left;
+        break;
+    }
     case TIFF_CFA_PATTERN_DIM:
         if (count != 2 || (ff_tget(&s->gb, type, s->le) != 2 &&
                            ff_tget(&s->gb, type, s->le) != 2)) {
-- 
2.17.1

